
#include "test.h"
#include "Widget.h"
#include <Windows.h>
#include <tchar.h>
#include <cassert>

namespace ghost{

	ATOM RegisterWndClass(
		HINSTANCE hInstance, 
		WNDPROC wndProc, 
		const TCHAR* className)
	{
		assert(className && wndProc);
		WNDCLASSEX wcex;
		wcex.cbSize = sizeof(wcex);
		wcex.style			= CS_HREDRAW | CS_VREDRAW;
		wcex.lpfnWndProc	= wndProc;
		wcex.cbClsExtra		= 0;
		wcex.cbWndExtra		= 0;
		wcex.hInstance		= hInstance;
		wcex.hIcon			= NULL;
		wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
		wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
		wcex.lpszMenuName	= NULL;
		wcex.lpszClassName	= className;
		wcex.hIconSm		= NULL;
		return ::RegisterClassEx(&wcex);
	}

	HWND CreateWnd(
		HINSTANCE hInstance, 
		const TCHAR* className, 
		const TCHAR* title)
	{
		assert(className && title);
		HWND hWnd = ::CreateWindowEx(0, className, title, WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
			CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);
		return hWnd;
	}

	void ShowWnd(HWND hWnd, int cmdShow)
	{
		assert(::IsWindow(hWnd));
		::ShowWindow(hWnd, cmdShow);
		::UpdateWindow(hWnd);
	}
	int MessageLoop()
	{
		MSG msg = {0};
		while (::GetMessage(&msg, NULL, 0, 0))
		{
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
		return msg.wParam;
	}
	LRESULT CALLBACK WndProc(
		HWND hWnd, 
		UINT message, 
		WPARAM wParam, 
		LPARAM lParam)
	{
		switch (message)
		{
		case WM_NCDESTROY:
			::PostQuitMessage(EXIT_SUCCESS);
			break;
		case WM_ERASEBKGND: // ֹ˸
			return S_OK;
		}
		return ::DefWindowProcA(hWnd, message, wParam, lParam);
	}

} // namespace ghost

int APIENTRY _tWinMain(
	HINSTANCE hInstance,
	HINSTANCE /*hPrevInstance*/,
	LPTSTR /*cmdLine*/,
	int cmdShow)
{
	static const TCHAR* const wndClassName = TEXT("WidgetTestWindow");
	static const TCHAR* const wndTitle = TEXT("WidgetTestWindow");
	if (0 == ghost::RegisterWndClass(hInstance, ghost::WndProc, wndClassName))
	{
		::GetLastError();
		assert(false);
		return EXIT_FAILURE;
	}
	HWND hWnd = ghost::CreateWnd(hInstance, wndClassName, wndTitle);
	if (!::IsWindow(hWnd))
	{
		::GetLastError();
		assert(false);
		return EXIT_FAILURE;
	}
	ghost::ShowWnd(hWnd, cmdShow);

	auto pRootWidget = ghost::Widget::Create(hWnd);

	// Բ33пؼߴΪ50*50
	int widgetWidth = 50;
	int widgetHeight = 50;
	RECT widgetRect = {10, 10, 10+widgetWidth, 10+widgetHeight};

	for (int i = 0; i < 3; ++i)
	{
		for (int j = 0; j < 3; ++j)
		{
			auto pWidget = ghost::Widget::Create(pRootWidget);
			pWidget->SetAbsoluteRect(widgetRect);

			::OffsetRect(&widgetRect, widgetWidth+5, 0);
		}
		::OffsetRect(&widgetRect, 10-widgetRect.left, widgetHeight+5);
	}

	return ghost::MessageLoop();
}